package cz.vsb.fei.lab;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import lombok.extern.log4j.Log4j2;

/**
 * Class <b>App</b> - main class
 * 
 * @author Java I
 */
@Log4j2
public class App {

	public static void main(String[] args) throws FileNotFoundException {
		log.info("Launching Java application.");
		App app = new App();
		app.printScores(app.loadScores());
		List<Score> scores = app.generateScores(200);
		app.saveScores(scores, "test.csv");
		app.saveScores(scores, "test/test.csv");
		try {
			app.printScores(app.loadScoresFromStream(new InputStreamReader(App.class.getResourceAsStream("/bestScore-ok.csv"))));
			app.printScores(app.loadScoresFromStream(new InputStreamReader(App.class.getResourceAsStream("/bestScores-err.csv"))));
		} catch (ScoreNotLoaded e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public List<Score> generateScores(int count) {
		List<Score> scores = new ArrayList<Score>();
		for (int i = 0; i < count; i++) {
			scores.add(Score.generate());
		}
		return scores;
	}

	public void saveScores(List<Score> scores, String file) throws FileNotFoundException {
		Paths.get(file).toAbsolutePath().getParent().toFile().mkdirs();
		try (PrintWriter printWriter = new PrintWriter(file)) {
			for (Score score : scores) {
				printWriter.format("%s;%d%n", score.getName(), score.getPoints());
			}
		}
	}

	public void printScores(List<Score> scores) {
		for (Score score : scores) {
			System.out.println(score);
		}
	}

	public List<Score> loadScores() {
		try {
			List<Score> scores = loadScoresFromFile("bestScores.csv");
			return scores;

		} catch (ScoreNotLoaded e) {
			System.out.println("Exception \"" + e.getMessage() + "\". alreaded loaded scores:" + e.getCount());
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return Collections.emptyList();
	}

	public List<Score> loadScoresFromFile(String filename) throws ScoreNotLoaded, FileNotFoundException, IOException {
		try (Reader inputReader = new FileReader(filename)) {
			return loadScoresFromStream(inputReader);
		} catch (ScoreNotLoaded e) {
			throw new ScoreNotLoaded("Info about scores from file " + filename + "cannot be loaded.", e.getCount(), e);
		}
	}

	public List<Score> loadScoresFromStream(Reader inputReader) throws ScoreNotLoaded {
		int count = 0;
		try (BufferedReader bufferedReader = new BufferedReader(inputReader)) {
			String line;
			List<Score> scores = new ArrayList<>();
			while ((line = bufferedReader.readLine()) != null) {
				String[] parts = line.split(";");
				try {
					scores.add(new Score(parts[0], Integer.parseInt(parts[1])));
					count++;
				} catch (NumberFormatException e) {
					System.out.println(
							String.format("WARNING: Cannot parse points for player %s. Value: %s", parts[0], parts[1]));
				}
			}
			return scores;
		} catch (Exception e) {
			throw new ScoreNotLoaded("Info about scores cannot be loaded.", count, e);
		}
	}

}